@@ -32,6 +32,16 @@ import stat
 import os
 
 using_subversion = True
+post_setuptools_distribute_merge = False
+
+if post_setuptools_distribute_merge or sys.version_info >= (3,0):
+    virtualenv_pypi_string = "virtualenv"
+    distribute_string = "distribute"
+    setuptools_string = "setuptools"
+else:
+    virtualenv_pypi_string = "virtualenv<1.10"
+    distribute_string = "distribute<0.7"
+    setuptools_string = "setuptools==0.6c11"
 
 #
 # Working around error with PYTHONHOME
@@ -564,28 +574,32 @@ class Repository(object):
 
     def install_package(self, install, preinstall, dir):
         Repository._configureExecutables()
+        extra_env = {}
+        if os.path.basename(dir) == 'distribute':
+            extra_env['DONT_PATCH_SETUPTOOLS'] = 'True'
         if Repository.useEasyInstall:
             try:
-                return self.easy_install(install, preinstall, dir)
+                return self.easy_install(install, preinstall, dir, extra_env)
             except:
                 print("ERROR installing with easy_install; falling back on PIP")
-            return self.pip_install(install, preinstall, dir)
+            return self.pip_install(install, preinstall, dir, extra_env)
         else:
             try:
-                return self.pip_install(install, preinstall, dir)
+                return self.pip_install(install, preinstall, dir, extra_env)
             except:
                 print("ERROR installing with PIP; falling back on easy_install")
-            return self.easy_install(install, preinstall, dir)
+            return self.easy_install(install, preinstall, dir, extra_env)
 
-    def easy_install(self, install, preinstall, dir):
+    def easy_install(self, install, preinstall, dir, extra_env=None):
         try:
             if install:
                 if self.offline:
-                    self.run([self.python, 'setup.py', 'install'], dir=dir)
+                    self.run([self.python, 'setup.py', 'install'], 
+                             dir=dir, extra_env=extra_env)
                 else:
                     self.run( self.easy_install_path 
                               + Repository.easy_install_flags + [self.pypi], 
-                              dir=os.path.dirname(dir))
+                              dir=os.path.dirname(dir), extra_env=extra_env)
             elif preinstall:
                 if not os.path.exists(dir):
                     self.run(self.easy_install_path 
@@ -593,7 +607,7 @@ class Repository(object):
                              + [ '--exclude-scripts', '--always-copy', 
                                  '--editable', '--build-directory', '.', 
                                  self.pypi], 
-                             dir=os.path.dirname(dir))
+                             dir=os.path.dirname(dir), extra_env=extra_env)
         except OSError:
             err,tb = sys.exc_info()[1:3] # BUG?
             print("")
@@ -608,22 +622,23 @@ class Repository(object):
                 sys.exit(1)
             print("Not aborting installer...")
 
-    def pip_install(self, install, preinstall, dir):
+    def pip_install(self, install, preinstall, dir, extra_env=None):
         try:
             if install:
                 if self.offline:
-                    self.run([self.python, 'setup.py', 'install'], dir=dir)
+                    self.run([self.python, 'setup.py', 'install'], 
+                             dir=dir, extra_env=extra_env)
                 else:
                     self.run( self.pip_path + ['install'] 
                               + Repository.pip_flags + [self.pypi],
-                              dir=os.path.dirname(dir))
+                              dir=os.path.dirname(dir), extra_env=extra_env)
             elif preinstall:
                 if not os.path.exists(dir):
                     self.run( self.pip_path + ['install'] 
                               + Repository.pip_flags 
                               + [ '--no-install', '--ignore-installed',
                                   '--build', '.', self.pypi ],
-                              dir=os.path.dirname(dir))
+                              dir=os.path.dirname(dir), extra_env=extra_env)
         except OSError:
             err,tb = sys.exc_info()[1:3] # BUG?
             print("")
@@ -652,7 +667,7 @@ class Repository(object):
         self.run( self.pip_path + ['install'] + Repository.pip_flags 
                   + ['--upgrade', self.pypi] )
 
-    def run(self, cmd, dir=None):
+    def run(self, cmd, dir=None, extra_env=None):
         cwd=os.getcwd()
         if not dir is None:
             os.chdir(dir)
@@ -660,7 +675,7 @@ class Repository(object):
         print( "\nRunning command '%s'\n\tin directory %s" 
                % (" ".join(cmd), cwd) )
         sys.stdout.flush()
-        call_subprocess(cmd, filter_stdout=filter_python_develop, show_stdout=True)
+        call_subprocess(cmd, filter_stdout=filter_python_develop, show_stdout=True, extra_env=extra_env)
         if not dir is None:
             os.chdir(cwd)
 
@@ -1002,6 +1017,16 @@ class Installer(object):
         os.environ['HTTP_PROXY'] = proxy
         os.environ['http_proxy'] = proxy
         print("  using the HTTP_PROXY environment: %s" % proxy)
+        proxy = ''
+        if not options.proxy is None:
+            proxy = options.proxy
+        if proxy is '':
+            proxy = os.environ.get('HTTPS_PROXY', '')
+        if proxy is '':
+            proxy = os.environ.get('https_proxy', '')
+        os.environ['HTTPS_PROXY'] = proxy
+        os.environ['https_proxy'] = proxy
+        print("  using the HTTPS_PROXY environment: %s" % proxy)
         print("")
         #
         # Disable the PYTHONPATH, to isolate this installation from
@@ -1068,13 +1093,21 @@ class Installer(object):
         print("-----------------------------------------------------------------")
         #
         if not options.preinstall and not (options.trunk or options.release):
-            self.sw_packages.insert( 0, Repository('virtualenv', pypi='virtualenv') )
+            self.sw_packages.insert( 0, Repository('virtualenv', pypi=virtualenv_pypi_string) )
             self.sw_packages.insert( 0, Repository('pip', pypi='pip') )
             #if sys.version_info[:2] < (3,0):
             #    self.sw_packages.insert( 0, Repository('setuptools', pypi='setuptools') )
             #else:
             #    self.sw_packages.insert( 0, Repository('distribute', pypi='distribute') )
-            self.sw_packages.insert( 0, Repository('distribute', pypi='distribute') )
+
+            # For a while, distribute was more reliable; however, in
+            # 4/2013, setuptools and distribute re-merged, and the
+            # current distribute is a shell wrapper -- that among other
+            # things -- no longer bundles the cli-*.exe binaries.
+            if post_setuptools_distribute_merge:
+                self.sw_packages.insert( 0, Repository('setuptools', pypi=setuptools_string) )
+            else:
+                self.sw_packages.insert( 0, Repository('distribute', pypi=distribute_string) )
             #
             # Configure the package versions, for offline installs
             #
@@ -1107,18 +1140,29 @@ class Installer(object):
         #
         # Get source packages
         #
-        self.sw_packages.insert( 0, Repository('virtualenv', pypi='virtualenv') )
+        self.sw_packages.insert( 0, Repository('virtualenv', pypi=virtualenv_pypi_string) )
         self.sw_packages.insert( 0, Repository('pip', pypi='pip') )
         #if sys.version_info[:2] < (3,0):
         #    self.sw_packages.insert( 0, Repository('setuptools', pypi='setuptools') )
         #else:
-        self.sw_packages.insert( 0, Repository('distribute', pypi='distribute') )
+
+        # For a while, distribute was more reliable; however, in
+        # 4/2013, setuptools and distribute re-merged, and the
+        # current distribute is a shell wrapper -- that among other
+        # things -- no longer bundles the cli-*.exe binaries.
+        if post_setuptools_distribute_merge:
+            self.sw_packages.insert( 0, Repository('setuptools', pypi=setuptools_string) )
+        else:
+            self.sw_packages.insert( 0, Repository('distribute', pypi=distribute_string) )
+
         if options.preinstall:
             #
             # When preinstalling, add the setuptools package to the installation list
             #
-            self.sw_packages.insert( 0, Repository('distribute', pypi='distribute') )
-            #self.sw_packages.insert( 0, Repository('setuptools', pypi='setuptools') )
+            #self.sw_packages.insert( 0, Repository('setuptools',pypi='setuptools') )
+            #self.sw_packages.insert( 0, Repository('distribute', pypi='distribute') )
+            pass
+
         for _pkg in options.packages:
             if os.path.exists(_pkg):
                 self.sw_packages.append( Repository(_pkg, local=os.path.abspath(_pkg)) )
@@ -1403,20 +1447,24 @@ def main():
 # This is a monkey patch, to control the execution of the install_setuptools()
 # function that is defined by virtualenv.
 #
-default_install_setuptools = install_setuptools
+try:
+    default_install_setuptools = install_setuptools
+except:
+    default_install_setuptools = None
 
 
 def install_setuptools(py_executable, unzip=False,
                        search_dirs=None, never_download=False):
     try:
         if install_setuptools.use_default:
-            #
-            # Always used 'distribute' instead of 'setuptools'
-            #
-            default_install_distribute(py_executable, unzip, search_dirs, never_download)
+            if post_setuptools_distribute_merge:
+                default_install_setuptools(py_executable, unzip, search_dirs, never_download)
+            else:
+                default_install_distribute(py_executable, unzip, search_dirs, never_download)
     except OSError:
         print("-----------------------------------------------------------------")
-        print("Error installing the 'distribute' package!")
+        print( "Error installing the '%s' package!" % 
+               ( 'setuptools' if post_setuptools_distribute_merge else 'distribute') )
         if os.environ['HTTP_PROXY'] == '':
             print("")
             print("WARNING: you may need to set your HTTP_PROXY environment variable!")
@@ -1430,7 +1478,10 @@ install_setuptools.use_default=True
 # This is a monkey patch, to control the execution of the install_distribute()
 # function that is defined by virtualenv.
 #
-default_install_distribute = install_distribute
+try:
+    default_install_distribute = install_distribute
+except:
+    default_install_distribute = None
 
 
 def install_distribute(py_executable, unzip=False,
@@ -1454,7 +1505,10 @@ install_distribute.use_default=True
 # This is a monkey patch, to control the execution of the install_pip()
 # function that is defined by virtualenv.
 #
-default_install_pip = install_pip
+try:
+    default_install_pip = install_pip
+except:
+    default_install_pip = None
 
 def install_pip(*args, **kwds):
     try:
@@ -1488,6 +1542,18 @@ def mkdir(path):
     else:
         logger.info('Directory %s already exists', path)
 
+
+#
+# This is a monkey patch to correc the capitalization of "true" set by
+# virtualenv
+#
+default_call_subprocess = call_subprocess
+def call_subprocess(cmd, **kwds):
+    if kwds.get('extra_env',{}) and \
+            kwds.get('extra_env',{}).get('DONT_PATCH_SETUPTOOLS', '') == 'true':
+        kwds['extra_env']['DONT_PATCH_SETUPTOOLS'] = 'True'
+    return default_call_subprocess(cmd, **kwds)
+
 #
 # The following methods will be called by virtualenv
 #
